@@ -1,4 +1,4 @@
-# $Id: Directory.pm 40158 2013-09-13 13:06:28Z anton $
+# $Id: Directory.pm 40407 2013-10-11 09:43:39Z anton $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/libpwdmodifier/lib/UvT/PwdModifier/Directory.pm $
 
 use strict;
@@ -15,6 +15,8 @@ field 'ldapEntry';
 field 'cf';
 field 'ldap';
 field debug => 0;
+field usePagedResults => 0;
+
 
 sub orgstats {
 	my %res;
@@ -119,6 +121,8 @@ sub stuff {
 }
 
 package UvT::PwdModifier::Directory;
+use Net::LDAP::Control::Paged;
+use Net::LDAP::Constant qw( LDAP_CONTROL_PAGED ); 
 
 use Data::Dumper;
 use Carp qw(cluck carp confess);
@@ -135,11 +139,12 @@ field 'capath';
 field 'bindAccountFilter';
 field 'bindPassword';
 field debug => 0;
+field usePagedResults => 0;
 
 
 sub ldapError {
 	# spiffy stopt de eerste parameter in $self
-	warn "coming in LDAPERROR";
+	warn "coming in LDAPERROR" if $self->debug;
 	my $res = $self;
 	confess $res->error unless $res->code == 4;
 	return $res;
@@ -156,8 +161,7 @@ sub ldap_connection {
 		my $retries = $cfhash->{'ldapRetries'} || $self->retries || 1;
 		my $state;
 		my $debug = $self->debug;
-		warn "debug: $debug";
-
+		warn "debug: $debug" if $debug;
 		warn "Connecting to ldapserver: $server" if $self->debug;
 		$ldap = new Net::LDAP($server,
 							  # onerror is o.a. NODIG om te reageren op foutieve authenticatie         
@@ -275,7 +279,7 @@ sub getAccount {
 		unless $entry;
 
 	return new UvT::PwdModifier::Directory::Account
-		(ldapEntry => $entry, cf => $self->cf, ldap => $self, debug => $self->debug), 
+		(ldapEntry => $entry, cf => $self->cf, ldap => $self, debug => $self->debug, usePagedResults => $self->usePagedResults), 
 		undef;
 }
 
@@ -304,10 +308,7 @@ sub getLdapEntriesByFilter {
 	my $ldap = $self->ldap_connection();
 
 	warn "getLdapEntriesByFilter; filter: $filter" if $self->debug;
-	my $res = $ldap->search(base => $base, filter => $filter, attrs => $attrs);
-	my @entries = $res->entries;
-#	warn Dumper ($res);
-	return \@entries;
+	my $entries =  $self->searchLdap(base => $base, filter => $filter, attrs => $attrs);
 }
 
 sub getUIDsByFilter {
@@ -336,6 +337,29 @@ sub listUnsuspended {
 	return $self->getUIDsByFilter($filter);
 }
 
+
+sub searchLdap {
+	my $ldap = $self->ldap_connection();
+	my @aggregatedEntries = ();
+
+	if ($self->usePagedResults) {
+		my $page = Net::LDAP::Control::Paged->new( size => 500 );
+		while (1) {
+			my $search = $ldap->search(@_, contrl => [$page]);
+			push (@aggregatedEntries, $search->entries);
+			my ($resp) = $search->control( LDAP_CONTROL_PAGED ) or last;
+			my $cookie = $resp->cookie or last;
+			$page->cookie($cookie);
+		}
+		
+	} else {
+		my $search = $ldap->search(@_);
+		my @entries = $search->entries;
+		push(@aggregatedEntries, @entries);
+	}
+	return \@aggregatedEntries;
+}
+
 sub getEntriesByUID {
 	my $uid = shift;
 	warn "getLdapAccoutByUid: $uid" if $self->debug;
@@ -348,16 +372,14 @@ sub getEntriesByUID {
 			{equalityMatch => {attributeDesc => 'uid', assertionValue => $uid}},
 		'Net::LDAP::Filter')->as_string;
 
-	my $search = $ldap->search(base => $base, filter => $filter);
-	my @entries = $search->entries;
-
+	my $entries = $self->searchLdap(base => $base, filter => $filter);
 	die "no match found for filter: '$filter'"
-		unless @entries;
+		unless @$entries;
 
 	die "multiple entries found for filter: '$filter'"
-		if @entries > 1;
+		if @$entries > 1;
 
-	return $entries[0];
+	return @$entries[0];
 }
 
 
